/**
 * @file 处理任务
 * @author kaivean(kaisey2012@163.com)
 */

import ployFillTobBlob from './ployFillTobBlob';
import getOrientation from './getOrientation';

function extend(target, source, checkfalse) {
    source = source || {};

    for (var key in source) {
        if (source.hasOwnProperty(key)) {
            if (!checkfalse || source[key]) {
                target[key] = source[key];
            }
        }
    }

    return target;
}

function loadFileToImg(file, callback) {
    var URL = window.URL || window.webkitURL;
    var url = URL.createObjectURL(file);
    var img = new Image();
    img.onload = function () {
        callback(img);
    };
    img.src = url;
}

function handleCanvas(img, orientation, options, callback) {
    ployFillTobBlob();

    var width = img.naturalWidth || img.width;
    var height = img.naturalHeight || img.height;
    if (orientation && ('5678'.indexOf(orientation) > -1)) {
        width = img.naturalHeight || img.height;
        height = img.naturalWidth || img.width;
    }

    var canvas = document.createElement('canvas');
    var canvasH = height;
    var canvasW = width;
    var maxLen = options.maxLen;
    var resolution = options.resolution; // 如果图片大于1百万像素，计算压缩比并将大小压至100万以下
    var isSquare = options.isSquare;

    if (isSquare) {
        // 生成的图片是正方形
        var ratio = width / height;
        var squareLen = maxLen || Math.sqrt(resolution); // 压缩的边长
        if (ratio > 1) {
            width = squareLen;
            height = squareLen / ratio;
        }
        else {
            height = squareLen;
            width = squareLen * ratio;
        }
        canvasH = canvasW = squareLen;
    }
    else {
        var ratio = maxLen
            ? (width > height ? width : height) / maxLen
            : width * height / resolution;
        if (ratio > 1) {
            !maxLen && (ratio = Math.sqrt(ratio)); // 未指定最大边长，则按照分辨率开方
            width /= ratio;
            height /= ratio;
            canvasW = width;
            canvasH = height;
        }
        else {
            ratio = 1;
        }
    }

    var dx = (canvasW - width) / 2;
    var dy = (canvasH - height) / 2;
    canvas.width = canvasW;
    canvas.height = canvasH;
    var ctx = canvas.getContext('2d');
    ctx.clearRect(0, 0, canvasW, canvasH);

    ctx.save();
    switch (orientation) {
        case 3:
            ctx.rotate(180 * Math.PI / 180);
            ctx.drawImage(img, -width - dx, -height - dy, width, height);
            break;
        case 6:
            ctx.rotate(90 * Math.PI / 180);
            ctx.drawImage(img, dy, -width - dx, height, width);
            break;
        case 8:
            ctx.rotate(270 * Math.PI / 180);
            ctx.drawImage(img, -height - dy, dx, height, width);
            break;
        case 2:
            ctx.translate(width, 0);
            ctx.scale(-1, 1);
            ctx.drawImage(img, dx, dy, width, height);
            break;
        case 4:
            ctx.translate(width, 0);
            ctx.scale(-1, 1);
            ctx.rotate(180 * Math.PI / 180);
            ctx.drawImage(img, -width - dx, -height - dy, width, height);
            break;
        case 5:
            ctx.translate(width, 0);
            ctx.scale(-1, 1);
            ctx.rotate(90 * Math.PI / 180);
            ctx.drawImage(img, dy, -width - dx, height, width);
            break;
        case 7:
            ctx.translate(width, 0);
            ctx.scale(-1, 1);
            ctx.rotate(270 * Math.PI / 180);
            ctx.drawImage(img, -height - dy, dx, height, width);
            break;

        default:
            ctx.drawImage(img, dx, dy, width, height);
    }
    ctx.restore();

    var blobOpts = {
        dxPercent: dx / canvasW,
        dyPercent: dy / canvasH
    }

    // 进行压缩, 压缩率 0.75
    canvas.toBlob(function (newFile) {
        callback(newFile, canvasW, canvasH, blobOpts);
    }, options.fileType, options.quality);
}

    /**
     * 入口： 压缩图片
     *
     * @param {File} ofile 文件
     * @param {Function} ocallback 压缩后回调
     * @param {Object} options 压缩参数
     * @param {float} options.maxLen 压缩后图片的长边
     * @param {number} options.resolution width * height 默认100w，maxLen优先
     * @param {float} options.quality 压缩图片的质量（canvas toBlob的一个参数）// 注，指定参数后，压缩速度变慢，失败率高，默认不开启
     * @param {boolean} options.isSquare 压缩成正方形，// todo padding成正方形，调整图片方向。 默认false，等比压缩
     * @param {boolean} options.compressType 压缩后的图片数据类型，0:二进制 , 1:base64, 默认0
     * @param {boolean} options.limitSize 压缩后图片的最大size, 超过此值，直接使用原文件
     * @return {boolean} 是否压缩成功
     */
function compressImage(ofile, ocallback, options) {
    var file = ofile || {};
    var callback = ocallback || function () {};
    var fileType = file.type || '';

    // 粗过滤
    if (!/^image\//.test(fileType)) {
        return callback(ofile, 0);
    }

    var opts = {
        isSquare: false,
        compressType: 0,
        resolution: 1000000,
        quality: null,
        fileType: fileType || 'image/jpeg',
        limitSize: file.size
    }

    opts = extend(opts, options);

    loadFileToImg(file, function (img) {
        if (img) {
            // 图片需要压缩
            var width = img.naturalWidth || img.width;
            var height = img.naturalHeight || img.height;
            if ((opts.maxLen && ((width > height ? width : height) <= opts.maxLen)) || (width * height <= opts.resolution)) {
                callback(ofile, 0);
                return;
            }

            // 修复ios下压缩后图片显示不对
            getOrientation(file, function (orientation) {
                orientation = orientation || 1;
                if (orientation) {
                    handleCanvas(img, orientation, opts, function (newFile, width, height, blobOpts) {
                        // 新图比旧图还大，就用旧图
                        var toCompress =  newFile.size <= opts.limitSize;

                        if (!toCompress) {
                            callback(ofile, -1);
                            return;
                        }

                        var finalFile = newFile;
                        if (opts.compressType === 1) {
                            // 需要将二进制内容转化为base64
                            var fileReader = new FileReader();
                            fileReader.onload = function (e) {
                                var base64 = e.target.result;
                                // 去掉data:image前缀
                                if (base64.indexOf('data:image') === 0) {
                                    base64 = base64.split(',')[1];
                                }

                                finalFile =  encodeURIComponent(base64);
                                callback(finalFile, 1, blobOpts);

                            }
                            fileReader.readAsDataURL(newFile);
                        }
                        else {
                            callback(finalFile, 1, blobOpts);
                        }
                    });
                }
                else {
                    callback(ofile, 0);
                }
            });
        }
        else {
            callback(ofile, 0);
        }
    });
}

export default compressImage;
